/****************************************************************************
 * Grafik-LCD Treiber mit VRAM.
 *
 * (C) Andreas Kaiser 2008.
 *
 * Die Software darf frei kopiert und verndert werden, solange sie
 * nicht ohne meine ausdrckliche Erlaubnis fr kommerzielle Zwecke
 * eingesetzt wird.
 ****************************************************************************/

#ifndef _CONFIG_H
#define _CONFIG_H

#define F_OSC		F_CPU
#define DEBUG		1

#define UART_BAUD_RATE	38400

#define LCD_USE_TIMER	1		// 1: SCLK/SEN via Timer statt Software (SEN erforderlich)

#if 1 // Pinbelegung 2xVRAM Mega644

#define LCD_LP_PORT	D
#define LCD_LP_PIN	(1<<PD6)	// OC2B when avail
#define LCD_SEN_PORT	D		// LCD_SEN_xxx nur definieren wenn vorhanden
#define LCD_SEN_PIN	(1<<PD5)	// OC1A
#define LCD_M_PORT	B
#define LCD_M_PIN	(1<<PB4)
#define LCD_SCLK_PORT	B
#define LCD_SCLK_PIN	(1<<PB3)	// OC0A
#define LCD_VEN_PORT	B
#define LCD_VEN_PIN	(1<<PB2)
#define LCD_FLM_PORT	B
#define LCD_FLM_PIN	(1<<PB0)

#define VRAM_ADDR_PORT	A		// vollstndig belegt
#define VRAM_DATA_PORT	C		// vollstndig belegt
#define VRAM_CTRL_PORT	D		// PD0,1 von UART belegt, PD5,6 von Timern, Rest VRAM,LP
#define VRAM_CTRL_IDLE	(0xFF & ~LCD_LP_PIN) // Ruhezustand, UART RX pullup active
#define VRAM_RAS	(1<<PD3)
#define VRAM_CAS	(1<<PD2)
#define VRAM_DTOE	(1<<PD7)
#define VRAM_WBWE	(1<<PD4)

#else // Pinbelegung Basisversion

#define LCD_LP_PORT	D
#define LCD_LP_PIN	(1<<PD6)	// OC2B when avail
#define LCD_SCLK_PORT	D
#define LCD_SCLK_PIN	(1<<PD5)
#define LCD_VEN_PORT	B
#define LCD_VEN_PIN	(1<<PB1)
#define LCD_FLM_PORT	B
#define LCD_FLM_PIN	(1<<PB0)

#define VRAM_ADDR_PORT	C		// vollstndig belegt
#define VRAM_DATA_PORT	A		// vollstndig belegt
#define VRAM_CTRL_PORT	D		// PD0,1 von UART belegt, PD5,6 von Timern, Rest VRAM,LP,SC
#define VRAM_CTRL_IDLE	(0xFF & ~LCD_LP_PIN & ~LCD_SCLK_PIN) // Ruhezustand, UART RX pullup active
#define VRAM_RAS	(1<<PD3)
#define VRAM_CAS	(1<<PD2)
#define VRAM_DTOE	(1<<PD7)
#define VRAM_WBWE	(1<<PD4)

#endif

/*****************************************************************************/

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "util/delay.h"

typedef uint8_t	uword_t; // most efficient machine word, 8bit min.
typedef int8_t	sword_t;
typedef uint8_t	bool;

#define true	1
#define false	0

#define GLUE(a, b)	a##b
#define PORT(x)		GLUE(PORT, x)
#define PIN(x)		GLUE(PIN, x)
#define DDR(x)		GLUE(DDR, x)

#define setBits(port,mask)	do{ (port) |=  (mask); }while(0)
#define clrBits(port,mask)	do{ (port) &= ~(mask); }while(0)
#define tstBits(port,mask)	((port) & (mask))

#define PCIE(n)		GLUE(PCIE, n)
#define PCMSK(n)	GLUE(PCMSK, n)

#define INT(n)		GLUE(INT,n)
#define ISC0(n)		GLUE(ISC,n)##0
#define ISC1(n)		GLUE(ISC,n)##1
#define INT_vect(n)	GLUE(INT,n)##_vect

/*****************************************************************************/

#define LOCK	uint8_t sreg = SREG; cli()
#define UNLOCK	SREG = sreg

/*****************************************************************************/

#define LCD_COLS	320
#define LCD_ROWS	240
#define LCD_DOUBLE	0		// 1: D4-7 is top half, D0-3 is bottom half
#define LCD_BITS	4		// 4/8 bits per LCD word (=4 when LCD_DOUBLE=1)
#define VRAM_ROWS	256
#define VRAM_COLS	256
#define VRAM_BITS	4		// 4 or 8 bits wide
#define FRAME_RATE	70		// 70..100 = 60..41s/line
#define VRAM_REFRESH	4		// per line, 0..4
#define LCD_M_CYCLE	0		// toggle M every <n> lines, unless 0

typedef uint8_t		vram_row_t;
typedef uint8_t		vram_col_t;

typedef uint8_t		lcd_row_t;
typedef uint16_t	lcd_col_t;

#define GLCD_FONT_8_6	1		// enable fonts
#define GLCD_FONT_10_8	1
#define GLCD_FONT_12_8	1
#define GLCD_FONT_48_32	0

/*****************************************************************************/
#if DEBUG
  #include <stdio.h>
  #define PRINTF(s, ...) { static char __s[] PROGMEM = (s); \
			   printf_P(__s, ## __VA_ARGS__); }
#else
  #define PRINTF(s, ...)
#endif
/*****************************************************************************/

#endif //_CONFIG_H
