/****************************************************************************
 * Grafik-LCD Treiber mit VRAM.
 *
 * (C) Andreas Kaiser 2008.
 *
 * Die Software darf frei kopiert und verndert werden, solange sie
 * nicht ohne meine ausdrckliche Erlaubnis fr kommerzielle Zwecke
 * eingesetzt wird.
 ****************************************************************************/

#include "config.h"
#include "vram.h"

void
vram_init(void)
{
    DDR(VRAM_ADDR_PORT)  = 0xFF;
    DDR(VRAM_DATA_PORT)  = 0x00;
    PORT(VRAM_CTRL_PORT) = VRAM_RAS | VRAM_CAS | VRAM_DTOE | VRAM_WBWE;
    DDR(VRAM_CTRL_PORT) |= VRAM_RAS | VRAM_CAS | VRAM_DTOE | VRAM_WBWE;

    _delay_us(200);

#ifdef LCD_SEN_PIN // SCLK is gated by SEN and inverted
    // Initialize VRAM - LCD module is not yet initialized
    setBits(DDR(LCD_SCLK_PORT), LCD_SCLK_PIN);
    setBits(PORT(LCD_SEN_PORT), LCD_SEN_PIN);
    setBits(DDR(LCD_SEN_PORT),  LCD_SEN_PIN);
    for (uint8_t i = 0; i < 10; ++i) {
	vram_refresh1();				// RAS cycle
	clrBits(PORT(LCD_SCLK_PORT), LCD_SCLK_PIN);	// SC cycle
	setBits(PORT(LCD_SCLK_PORT), LCD_SCLK_PIN);
    }
#else
    // Initialize VRAM - LCD module is not yet initialized
    setBits(DDR(LCD_SCLK_PORT), LCD_SCLK_PIN);
    for (uint8_t i = 0; i < 10; ++i) {
	vram_refresh1();				// RAS cycle
	setBits(PORT(LCD_SCLK_PORT), LCD_SCLK_PIN);	// SC cycle
	clrBits(PORT(LCD_SCLK_PORT), LCD_SCLK_PIN);
    }
#endif
}

uint8_t
vram_read(vram_row_t row, vram_col_t col)
{
    return vram_read_i(row, col);
}

void
vram_write(vram_row_t row, vram_col_t col, uint8_t mask, uint8_t data)
{
    vram_write_i(row, col, mask, data);
}
